/*
 * color_win.c source file for extace (color picker)
 * 
 * /GDK/GNOME sound (esd) system output display program
 * 
 * Copyright (C) 1998 by Michael Fullbright
 * Re-hacked to look good by The Rasterman
 * 
 * Hacked some more by Dave J. Andruczyk <dave@techdev.buffalostate.edu>
 * to be fully scalable with lots of new options, for tilting the axis's
 * and various other cool stuff.
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include <config.h>
#include <globals.h>
#include <protos.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>
#include <math.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk_imlib.h>


// See globals.h for variable declarations and DEFINES

gint color_event (GtkWidget *widget, GdkEventButton *event)
{

    gint handled = FALSE;
    GtkWidget *colorsel;

    color_loc = widget->allocation.height - event->y;
    /* Check if we've received a button pressed event */
    
           if (event->type == GDK_BUTTON_PRESS && colorseldlg == NULL)
	   {
	       /* Yes, we have an event and there's no colorseldlg yet! */
	       colorseldlg = gtk_color_selection_dialog_new("Select color");

	       /* Get the ColorSelection widget */

	       colorsel = GTK_COLOR_SELECTION_DIALOG(colorseldlg)->colorsel;

	       /* Connect to the "color_changed" signal, set the client-data
		*          *             * to the colorsel widget */


	       gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colorseldlg)->ok_button), "clicked",(GtkSignalFunc)color_button, (gpointer)colorsel);
				  
	       gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colorseldlg)->cancel_button), "clicked",(GtkSignalFunc)color_button, (gpointer)colorsel);
	       /* Show the dialog */

	       gtk_widget_show(colorseldlg);
	   }

    return handled;

}
int color_button(GtkWidget *widget, gpointer data)
{
    if (GTK_BUTTON(GTK_COLOR_SELECTION_DIALOG(colorseldlg)->ok_button))
    {
	//g_print("OK buton pressed\n");
	color_changed_cb(NULL,GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(colorseldlg)->colorsel));
	update_gradient(NULL, color_loc);
	init_colortab();
	gtk_widget_destroy(colorseldlg);
	colorseldlg=NULL;
    }
    else
    {
	gtk_widget_destroy(colorseldlg);
	        colorseldlg=NULL;
    }
    return TRUE;
}
void color_changed_cb (GtkWidget *widget, GtkColorSelection *colorsel)
{
    gdouble color[3];
    GdkColormap *colormap;

    /* Get drawingarea colormap */

    colormap = gdk_window_get_colormap (color_win->window);

    /* Get current color */

    gtk_color_selection_get_color (colorsel,color);

    /* Fit to a unsigned 16 bit integer (0..65535) and
     *      *         * insert into the GdkColor structure */

    temp_color.red = (guint16)(color[0]*255.0);
    temp_color.green = (guint16)(color[1]*255.0);
    temp_color.blue = (guint16)(color[2]*255.0);

}
void update_gradient(GtkWidget *widget, int y)
{
//    y = MAXBANDS-y;
    if (y <= 16) // start color
	start = temp_color;
    if ((y > 16) && (y <= 48)) // part 2 color
	pt2 = temp_color;
    if ((y > 48) && (y <= 80)) // part 3 color
	pt3 = temp_color;
    if ((y > 80) && (y <= 112)) // part 4 color
	pt4 = temp_color;
    if (y > 112) // end color
	end = temp_color;
}

void init_colortab(void)
{
    gint i,j;
    gint w, h;
    gint r,g,b;

    gint cr[MAXBANDS],cg[MAXBANDS],cb[MAXBANDS];
    unsigned char *data;

    j=0;
    for(i=0;i<32;i++)
    {
	cr[j]=(((31-i)*start.red)+(i*pt2.red))/31;
	cg[j]=(((31-i)*start.green)+(i*pt2.green))/31;
	cb[j]=(((31-i)*start.blue)+(i*pt2.blue))/31;
	j++;
    }
    for(i=0;i<32;i++)
    {
	cr[j]=(((31-i)*pt2.red)+(i*pt3.red))/31;
	cg[j]=(((31-i)*pt2.green)+(i*pt3.green))/31;
	cb[j]=(((31-i)*pt2.blue)+(i*pt3.blue))/31;
	j++;
    }
    for(i=0;i<32;i++)
    {
	cr[j]=(((31-i)*pt3.red)+(i*pt4.red))/31;
	cg[j]=(((31-i)*pt3.green)+(i*pt4.green))/31;
	cb[j]=(((31-i)*pt3.blue)+(i*pt4.blue))/31;
	j++;
    }
    for(i=0;i<32;i++)
    {
	cr[j]=(((31-i)*pt4.red)+(i*end.red))/31;
	cg[j]=(((31-i)*pt4.green)+(i*end.green))/31;
	cb[j]=(((31-i)*pt4.blue)+(i*end.blue))/31;
	j++;
    }
    for(i=0;i<bands;i++)
    {
	for(j=0;j<MAXBANDS;j++)
	{
	    r=cr[j]+((i-16)*2);
	    g=cg[j]+((i-16)*2);
	    b=cb[j]+((i-16)*2);
	    if (r<0) r=0;
	    else if (r>255) r=255;
	    if (g<0) g=0;
	    else if (g>255) g=255;
	    if (b<0) b=0;
	    else if (b>255) b=255;
	    colortab[i][j]=gdk_imlib_best_color_match(&r,&g,&b);
	}
    }
    data=malloc(MAXBANDS*3);
    for(i=0;i<MAXBANDS;i++)
    {
	data[(i*3)]=cr[(MAXBANDS-1)-i];
	data[(i*3)+1]=cg[(MAXBANDS-1)-i];
	data[(i*3)+2]=cb[(MAXBANDS-1)-i];
    }
    im=gdk_imlib_create_image_from_data(data,NULL,1,MAXBANDS);
    free(data);
    w=MAXBANDS;
    h=im->rgb_height;

    for(i=128;i<256;i++)
    {
	gdk_imlib_render(im,1,i-127);
	grad[i]=gdk_imlib_move_image(im);
    }
    gdk_imlib_flip_image_vertical(im);
    for(i=0;i<127;i++)
    {
	gdk_imlib_render(im,1,127-i);
	grad[i]=gdk_imlib_move_image(im);
    }
    grad_win();
    gdk_imlib_kill_image(im);
}

void grad_win()
{
    int h = MAXBANDS;
    int w = MAXBANDS*1.25;

    if(!color_win)
    {
	color_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(color_win),"Color Picker");
	gtk_widget_set_usize(color_win,w,h);
	gtk_window_set_policy(GTK_WINDOW(color_win), FALSE,   // allow shrink
			      FALSE,         // allow grow
			      FALSE);       // auto shrink
	gtk_signal_connect(GTK_OBJECT(color_win),"delete_event",
			   GTK_SIGNAL_FUNC(close_color_win),NULL);
	gtk_widget_set_events (color_win, GDK_BUTTON_PRESS_MASK);
	gtk_signal_connect(GTK_OBJECT(color_win), "button_press_event",
			   (GtkSignalFunc) color_event, NULL);
			       

	grad_disp = gtk_drawing_area_new();
	gtk_container_add(GTK_CONTAINER(color_win), grad_disp);
	gtk_widget_show(grad_disp);
	gtk_widget_realize(grad_disp);

	gtk_widget_show_all(color_win);
    }
    if (grad_pixmap)
	gdk_pixmap_unref(grad_pixmap);

    grad_pixmap = gdk_pixmap_new(grad_disp->window,width,height,
				 gtk_widget_get_visual(grad_disp)->depth);

    gdk_imlib_flip_image_vertical(im);
    gdk_imlib_render(im,w,h);
    grad_pixmap=gdk_imlib_move_image(im);

    gdk_window_set_back_pixmap(grad_disp->window,grad_pixmap,0);

    gdk_draw_pixmap(grad_disp->window,
		    grad_disp->style->fg_gc[GTK_WIDGET_STATE (grad_disp)],
		    grad_pixmap,
		    0,0,
		    0,0,
		    w,h);

}
